# Generated by roxygen2: do not edit by hand

S3method(determine_dims,sfc)
S3method(determine_dims,sfg)
S3method(has_m,sfc)
S3method(has_m,sfg)
S3method(has_z,sfc)
S3method(has_z,sfg)
S3method(print,PortalGroup)
S3method(print,PortalItem)
S3method(print,PortalUser)
S3method(print,arc_gp_job)
export("%||%")
export(arc_agent)
export(arc_base_req)
export(arc_form_params)
export(arc_gp_job)
export(arc_group)
export(arc_group_content)
export(arc_group_users)
export(arc_host)
export(arc_item)
export(arc_item_data)
export(arc_job_status)
export(arc_paginate_req)
export(arc_portal_resources)
export(arc_portal_self)
export(arc_portal_servers)
export(arc_portal_urls)
export(arc_portal_users)
export(arc_self_meta)
export(arc_token)
export(arc_url_parse)
export(arc_url_type)
export(arc_user)
export(arc_user_content)
export(arc_user_self)
export(as_esri_features)
export(as_esri_featureset)
export(as_esri_geometry)
export(as_extent)
export(as_feature_collection)
export(as_features)
export(as_featureset)
export(as_fields)
export(as_form_params)
export(as_gp_areal_unit)
export(as_gp_date)
export(as_gp_feature_record_set)
export(as_gp_linear_unit)
export(as_gp_raster_layer)
export(as_layer)
export(as_layer_definition)
export(as_record_set)
export(as_spatial_reference)
export(auth_binding)
export(auth_client)
export(auth_code)
export(auth_key)
export(auth_shiny)
export(auth_user)
export(catch_error)
export(check_dots_named)
export(check_token_has_user)
export(compact)
export(data_frame)
export(date_to_ms)
export(detect_errors)
export(determine_dims)
export(determine_esri_geo_type)
export(fetch_layer_metadata)
export(fields_as_ptype_df)
export(from_envelope)
export(from_esri_date)
export(from_spatial_reference)
export(gp_areal_unit)
export(gp_job_from_url)
export(gp_linear_unit)
export(has_m)
export(has_z)
export(infer_esri_type)
export(is_date)
export(is_url)
export(item_keyword)
export(item_type)
export(new_gp_job)
export(oauth_provider_arcgis)
export(obj_check_token)
export(parse_esri_json)
export(parse_gp_areal_unit)
export(parse_gp_date)
export(parse_gp_feature_record_set)
export(parse_gp_linear_unit)
export(parse_gp_record_set)
export(parse_spatial_reference)
export(portal_item_keywords)
export(portal_item_types)
export(ptype_tbl)
export(rbind_results)
export(refresh_token)
export(search_items)
export(set_arc_token)
export(unset_arc_token)
export(validate_crs)
export(validate_or_refresh_token)
if (getRversion() < "4.3.0") importFrom("S7", "@")
importFrom(R6,R6Class)
importFrom(lifecycle,deprecated)
useDynLib(arcgisutils, .registration = TRUE)
