# Generated by extendr: Do not edit by hand

# nolint start

#
# This file was created with the following call:
#   .Call("wrap__make_arcgisutils_wrappers", use_symbols = TRUE, package_name = "arcgisutils")

#' @usage NULL
#' @useDynLib arcgisutils, .registration = TRUE
NULL

sfg_point_as_point <- function(x, sr) .Call(wrap__sfg_point_as_point, x, sr)

sfg_multipoint_as_multipoint <- function(x, sr) .Call(wrap__sfg_multipoint_as_multipoint, x, sr)

sfg_linestring_as_polyline <- function(x, sr) .Call(wrap__sfg_linestring_as_polyline, x, sr)

sfg_multilinestring_as_polyline <- function(x, sr) .Call(wrap__sfg_multilinestring_as_polyline, x, sr)

sfg_polygon_as_polygon <- function(x, sr) .Call(wrap__sfg_polygon_as_polygon, x, sr)

sfg_multipolygon_as_polygon <- function(x, sr) .Call(wrap__sfg_multipolygon_as_polygon, x, sr)

sf_as_features_2d_list <- function(attrs, geoms, n, sr) .Call(wrap__sf_as_features_2d_list, attrs, geoms, n, sr)

sf_as_features_2d_string <- function(attrs, geoms, n, sr) .Call(wrap__sf_as_features_2d_string, attrs, geoms, n, sr)

sf_as_features_3d_list <- function(attrs, geoms, n, sr) .Call(wrap__sf_as_features_3d_list, attrs, geoms, n, sr)

sf_as_features_3d_string <- function(attrs, geoms, n, sr) .Call(wrap__sf_as_features_3d_string, attrs, geoms, n, sr)

sfc_linestring_features_2d_list <- function(x, sr) .Call(wrap__sfc_linestring_features_2d_list, x, sr)

sfc_linestring_features_2d_string <- function(x, sr) .Call(wrap__sfc_linestring_features_2d_string, x, sr)

sfc_linestring_features_3d_list <- function(x, sr) .Call(wrap__sfc_linestring_features_3d_list, x, sr)

sfc_linestring_features_3d_string <- function(x, sr) .Call(wrap__sfc_linestring_features_3d_string, x, sr)

sfc_linestring_featureset_2d_list <- function(x, sr) .Call(wrap__sfc_linestring_featureset_2d_list, x, sr)

sfc_linestring_featureset_2d_string <- function(x, sr) .Call(wrap__sfc_linestring_featureset_2d_string, x, sr)

sfc_linestring_featureset_3d_list <- function(x, sr) .Call(wrap__sfc_linestring_featureset_3d_list, x, sr)

sfc_linestring_featureset_3d_string <- function(x, sr) .Call(wrap__sfc_linestring_featureset_3d_string, x, sr)

sfc_multilinestring_features_2d_string <- function(x, sr) .Call(wrap__sfc_multilinestring_features_2d_string, x, sr)

sfc_multilinestring_features_2d_list <- function(x, sr) .Call(wrap__sfc_multilinestring_features_2d_list, x, sr)

sfc_multilinestring_features_3d_string <- function(x, sr) .Call(wrap__sfc_multilinestring_features_3d_string, x, sr)

sfc_multilinestring_features_3d_list <- function(x, sr) .Call(wrap__sfc_multilinestring_features_3d_list, x, sr)

sfc_multilinestring_featureset_2d_string <- function(x, sr) .Call(wrap__sfc_multilinestring_featureset_2d_string, x, sr)

sfc_multilinestring_featureset_2d_list <- function(x, sr) .Call(wrap__sfc_multilinestring_featureset_2d_list, x, sr)

sfc_multilinestring_featureset_3d_string <- function(x, sr) .Call(wrap__sfc_multilinestring_featureset_3d_string, x, sr)

sfc_multilinestring_featureset_3d_list <- function(x, sr) .Call(wrap__sfc_multilinestring_featureset_3d_list, x, sr)

sfc_multipoint_features_2d_string <- function(x, sr) .Call(wrap__sfc_multipoint_features_2d_string, x, sr)

sfc_multipoint_features_2d_list <- function(x, sr) .Call(wrap__sfc_multipoint_features_2d_list, x, sr)

sfc_multipoint_features_3d_string <- function(x, sr) .Call(wrap__sfc_multipoint_features_3d_string, x, sr)

sfc_multipoint_features_3d_list <- function(x, sr) .Call(wrap__sfc_multipoint_features_3d_list, x, sr)

sfc_multipoint_featureset_2d_string <- function(x, sr) .Call(wrap__sfc_multipoint_featureset_2d_string, x, sr)

sfc_multipoint_featureset_2d_list <- function(x, sr) .Call(wrap__sfc_multipoint_featureset_2d_list, x, sr)

sfc_multipoint_featureset_3d_string <- function(x, sr) .Call(wrap__sfc_multipoint_featureset_3d_string, x, sr)

sfc_multipoint_featureset_3d_list <- function(x, sr) .Call(wrap__sfc_multipoint_featureset_3d_list, x, sr)

sfc_multipolygon_features_2d_string <- function(x, sr) .Call(wrap__sfc_multipolygon_features_2d_string, x, sr)

sfc_multipolygon_features_3d_string <- function(x, sr) .Call(wrap__sfc_multipolygon_features_3d_string, x, sr)

sfc_multipolygon_featureset_2d_string <- function(x, sr) .Call(wrap__sfc_multipolygon_featureset_2d_string, x, sr)

sfc_multipolygon_featureset_3d_string <- function(x, sr) .Call(wrap__sfc_multipolygon_featureset_3d_string, x, sr)

sfc_multipolygon_features_2d_list <- function(x, sr) .Call(wrap__sfc_multipolygon_features_2d_list, x, sr)

sfc_multipolygon_features_3d_list <- function(x, sr) .Call(wrap__sfc_multipolygon_features_3d_list, x, sr)

sfc_multipolygon_featureset_2d_list <- function(x, sr) .Call(wrap__sfc_multipolygon_featureset_2d_list, x, sr)

sfc_multipolygon_featureset_3d_list <- function(x, sr) .Call(wrap__sfc_multipolygon_featureset_3d_list, x, sr)

sfc_point_features_2d_string <- function(x, sr) .Call(wrap__sfc_point_features_2d_string, x, sr)

sfc_point_features_3d_string <- function(x, sr) .Call(wrap__sfc_point_features_3d_string, x, sr)

sfc_point_featureset_2d_string <- function(x, sr) .Call(wrap__sfc_point_featureset_2d_string, x, sr)

sfc_point_featureset_3d_string <- function(x, sr) .Call(wrap__sfc_point_featureset_3d_string, x, sr)

sfc_point_features_2d_list <- function(x, sr) .Call(wrap__sfc_point_features_2d_list, x, sr)

sfc_point_features_3d_list <- function(x, sr) .Call(wrap__sfc_point_features_3d_list, x, sr)

sfc_point_featureset_2d_list <- function(x, sr) .Call(wrap__sfc_point_featureset_2d_list, x, sr)

sfc_point_featureset_3d_list <- function(x, sr) .Call(wrap__sfc_point_featureset_3d_list, x, sr)

sfc_polygon_features_2d_string <- function(x, sr) .Call(wrap__sfc_polygon_features_2d_string, x, sr)

sfc_polygon_features_3d_string <- function(x, sr) .Call(wrap__sfc_polygon_features_3d_string, x, sr)

sfc_polygon_featureset_2d_string <- function(x, sr) .Call(wrap__sfc_polygon_featureset_2d_string, x, sr)

sfc_polygon_featureset_3d_string <- function(x, sr) .Call(wrap__sfc_polygon_featureset_3d_string, x, sr)

sfc_polygon_features_2d_list <- function(x, sr) .Call(wrap__sfc_polygon_features_2d_list, x, sr)

sfc_polygon_features_3d_list <- function(x, sr) .Call(wrap__sfc_polygon_features_3d_list, x, sr)

sfc_polygon_featureset_2d_list <- function(x, sr) .Call(wrap__sfc_polygon_featureset_2d_list, x, sr)

sfc_polygon_featureset_3d_list <- function(x, sr) .Call(wrap__sfc_polygon_featureset_3d_list, x, sr)

as_featureset_2d_list <- function(attrs, geoms, n, sr) .Call(wrap__as_featureset_2d_list, attrs, geoms, n, sr)

as_featureset_2d_string <- function(attrs, geoms, n, sr) .Call(wrap__as_featureset_2d_string, attrs, geoms, n, sr)

as_featureset_3d_list <- function(attrs, geoms, n, sr, has_z) .Call(wrap__as_featureset_3d_list, attrs, geoms, n, sr, has_z)

as_featureset_3d_string <- function(attrs, geoms, n, sr, has_z) .Call(wrap__as_featureset_3d_string, attrs, geoms, n, sr, has_z)


# nolint end
